Add support for Oxford OXPCIe 952 PCIe UART cards.

Signed-off-by: Stefan Reinauer <reinauer@google.com>

diff --git a/src/output.c b/src/output.c
index 936d3d8..526b92a 100644
--- a/src/output.c
+++ b/src/output.c
@@ -11,6 +11,7 @@
 #include "bregs.h" // struct bregs
 #include "config.h" // CONFIG_*
 #include "biosvar.h" // GET_GLOBAL
+#include "pci.h" // pci_find_device
 
 struct putcinfo {
     void (*func)(struct putcinfo *info, char c);
@@ -24,19 +25,39 @@ struct putcinfo {
 #define DEBUG_PORT PORT_SERIAL1
 #define DEBUG_TIMEOUT 100000
 
+#define OXPCIE_COM1 (oxpcie_bar + 0x1000)
+u32 oxpcie_bar VAR16VISIBLE;
+
 void
 debug_serial_setup(void)
 {
+    int oxpcie_bdf;
+    oxpcie_bar = NULL;
+
     if (!CONFIG_DEBUG_SERIAL)
         return;
-    // setup for serial logging: 8N1
+
+    oxpcie_bdf = pci_find_device(0x1415, 0xc158);
+    if (oxpcie_bdf != -1)
+        oxpcie_bar = pci_config_readl(oxpcie_bdf, 0x10);
+
     u8 oldparam, newparam = 0x03;
-    oldparam = inb(DEBUG_PORT+SEROFF_LCR);
-    outb(newparam, DEBUG_PORT+SEROFF_LCR);
-    // Disable irqs
     u8 oldier, newier = 0;
-    oldier = inb(DEBUG_PORT+SEROFF_IER);
-    outb(newier, DEBUG_PORT+SEROFF_IER);
+    if (oxpcie_bar) {
+        // setup for serial logging: 8N1
+        oldparam = readb((void *)OXPCIE_COM1+SEROFF_LCR);
+        writeb((void *)OXPCIE_COM1+SEROFF_LCR, newparam);
+        // Disable irqs
+        oldier = readb((void *)OXPCIE_COM1+SEROFF_IER);
+        writeb((void *)OXPCIE_COM1+SEROFF_IER, newier);
+        // setup for serial logging: 8N1
+    } else {
+        oldparam = inb(DEBUG_PORT+SEROFF_LCR);
+        outb(newparam, DEBUG_PORT+SEROFF_LCR);
+        // Disable irqs
+        oldier = inb(DEBUG_PORT+SEROFF_IER);
+        outb(newier, DEBUG_PORT+SEROFF_IER);
+    }
 
     if (oldparam != newparam || oldier != newier)
         dprintf(1, "Changing serial settings was %x/%x now %x/%x\n"
@@ -50,11 +71,19 @@ debug_serial(char c)
     if (!CONFIG_DEBUG_SERIAL)
         return;
     int timeout = DEBUG_TIMEOUT;
-    while ((inb(DEBUG_PORT+SEROFF_LSR) & 0x60) != 0x60)
-        if (!timeout--)
-            // Ran out of time.
-            return;
-    outb(c, DEBUG_PORT+SEROFF_DATA);
+    if (oxpcie_bar) {
+        while ((readb((void *)OXPCIE_COM1+SEROFF_LSR) & 0x60) != 0x60)
+            if (!timeout--)
+                // Ran out of time.
+                return;
+        writeb((void *)OXPCIE_COM1+SEROFF_DATA, c);
+    } else {
+        while ((inb(DEBUG_PORT+SEROFF_LSR) & 0x60) != 0x60)
+            if (!timeout--)
+                // Ran out of time.
+                return;
+        outb(c, DEBUG_PORT+SEROFF_DATA);
+    }
 }
 
 // Make sure all serial port writes have been completely sent.
@@ -64,10 +93,18 @@ debug_serial_flush(void)
     if (!CONFIG_DEBUG_SERIAL)
         return;
     int timeout = DEBUG_TIMEOUT;
-    while ((inb(DEBUG_PORT+SEROFF_LSR) & 0x40) != 0x40)
-        if (!timeout--)
-            // Ran out of time.
-            return;
+
+    if (oxpcie_bar) {
+        while ((readb((void *)OXPCIE_COM1+SEROFF_LSR) & 0x40) != 0x40)
+            if (!timeout--)
+                // Ran out of time.
+                return;
+    } else {
+        while ((inb(DEBUG_PORT+SEROFF_LSR) & 0x40) != 0x40)
+            if (!timeout--)
+                // Ran out of time.
+                return;
+    }
 }
 
 // Write a character to debug port(s).
